import bpy
from ..scripts import create_house


class create_house_operator(bpy.types.Operator):
    bl_idname = "object.house2024"
    bl_label = "> Create House <"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        print("Hello Blender")

        verts, edges, faces = create_house.create_house_mesh(
            context.scene.create_house.size, context.scene.create_house.roof
        )

        obj = self.create_mesh_object(context, verts, edges, faces)

        obj.data["data_type"] = "HOUSE"
        obj.data["has_chimney"] = False

        # Disable array
        # if context.scene.create_house.use_modifier:
        #     self.apply_array_modifier(context, obj)
        # else:
        #     self.apply_array_manually(context, obj)

        self.report({"INFO"}, "House created!")
        return {"FINISHED"}

    def create_mesh_object(self, context, verts, edges, faces):
        mesh = bpy.data.meshes.new("HouseMesh")
        mesh.from_pydata(verts, edges, faces)
        mesh.update()

        obj = bpy.data.objects.new("House", mesh)

        collection = bpy.context.collection
        collection.objects.link(obj)

        obj.select_set(True)
        context.view_layer.objects.active = obj

        return obj

    def apply_array_modifier(self, context, obj):
        array_mod = obj.modifiers.new(name="ArrayModifier", type="ARRAY")
        array_mod.count = context.scene.create_house.array_count

        array_mod.use_relative_offset = True
        array_mod.relative_offset_displace = context.scene.create_house.array_offset

    def apply_array_manually(self, context, obj):
        offset_vector = (
            context.scene.create_house.array_offset * context.scene.create_house.size
        )

        for i in range(1, context.scene.create_house.array_count):
            obj_duplicate = obj.copy()
            obj_duplicate.data = obj.data.copy()

            obj_duplicate.location = obj.location + (offset_vector * i)

            context.collection.objects.link(obj_duplicate)


class load_houses(bpy.types.Operator):
    bl_idname = "object.load_houses"
    bl_label = "> Load Houses <"

    def execute(self, context):

        context.scene.house_group.coll.clear()

        for obj in context.scene.objects:
            if "data_type" in obj.data.keys() and obj.data["data_type"] == "HOUSE":
                item = context.scene.house_group.coll.add()
                item.name = obj.name
                item.has_chimney = obj.data["has_chimney"]

        return {"FINISHED"}
